@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX process handler.
 ;;; yatexprc.el
-;;; (c )1993-2003 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Thu May  1 22:37:31 2003 on firestorm
-;;; $Id: yatexprc.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
+;;; (c)1993-2009 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Sep 28 10:47:11 2009 on firestorm
+;;; $Id: yatexprc.el,v 1.74 2009/09/28 01:54:43 yuuji Rel $
 
 ;(require 'yatex)
 (require 'yatexlib)
@@ -27,15 +27,24 @@
   "Shell option for command execution.")
 
 (defvar YaTeX-latex-message-code
+;;   (cond
+;;    (YaTeX-dos (cdr (assq 1 YaTeX-kanji-code-alist)))
+;;    ((and YaTeX-emacs-20 (member 'undecided (coding-system-list))
+;; 	 'undecided))
+;;    ((featurep 'mule)
+;;     (or (and (boundp '*autoconv*) *autoconv*)
+;; 	(and (fboundp 'coding-system-list) 'automatic-conversion)))
+;;    ((boundp 'NEMACS)
+;;     (cdr (assq (if YaTeX-dos 1 2) YaTeX-kanji-code-alist))))
   (cond
    (YaTeX-dos (cdr (assq 1 YaTeX-kanji-code-alist)))
-   ((and YaTeX-emacs-20 (member 'undecided (coding-system-list))
-	 'undecided))
-   ((featurep 'mule)
-    (or (and (boundp '*autoconv*) *autoconv*)
-	(and (fboundp 'coding-system-list) 'automatic-conversion)))
+   (YaTeX-emacs-20
+    (cdr (assoc latex-message-kanji-code YaTeX-kanji-code-alist)))
+   ((boundp 'MULE)
+    (symbol-value
+     (cdr (assoc latex-message-kanji-code YaTeX-kanji-code-alist))))
    ((boundp 'NEMACS)
-    (cdr (assq (if YaTeX-dos 1 2) YaTeX-kanji-code-alist))))
+    latex-message-kanji-code))
   "Process coding system for LaTeX.")
 
 (if YaTeX-typeset-buffer-syntax nil
@@ -304,12 +313,19 @@ action wants to be done, A:Add list, R:Replace list, %:comment-out list."
 (defvar YaTeX-call-command-history nil
   "Holds history list of YaTeX-call-command-on-file.")
 (put 'YaTeX-call-command-history 'no-default t)
-(defun YaTeX-call-command-on-file (base-cmd buffer)
+(defun YaTeX-call-command-on-file (base-cmd buffer &optional file)
+  "Call external command BASE-CMD int the BUFFER.
+By default, pass the basename of current file.  Optional 3rd argument
+FILE changes the default file name."
   (YaTeX-save-buffers)
   (YaTeX-typeset
    (read-string-with-history
     "Call command: "
-    (concat base-cmd " " (YaTeX-get-preview-file-name))
+    (concat base-cmd " "
+	    (let ((me (file-name-nondirectory (or file buffer-file-name))))
+	      (if (string-match "\\.tex" me)
+		  (substring me 0 (match-beginning 0))
+		me)))
     'YaTeX-call-command-history)
    buffer))
 
@@ -317,7 +333,9 @@ action wants to be done, A:Add list, R:Replace list, %:comment-out list."
   "Pass the bibliography data of editing file to bibtex."
   (interactive)
   (YaTeX-save-buffers)
-  (YaTeX-call-command-on-file cmd "*YaTeX-bibtex*" ))
+  (let ((main (or YaTeX-parent-file
+		  (progn (YaTeX-visit-main t) buffer-file-name))))
+    (YaTeX-call-command-on-file cmd "*YaTeX-bibtex*" main)))
 
 (defun YaTeX-kill-typeset-process (proc)
   "Kill process PROC after sending signal to PROC.
@@ -395,11 +413,11 @@ PROC should be process identifier."
       (concat
        (catch 'found-paper
 	 (mapcar (lambda (pair)
-		   (if (member (car pair) opts)
+		   (if (YaTeX-member (car pair) opts)
 		       (throw 'found-paper (cdr pair))))
 		 YaTeX-paper-type-alist)
 	 YaTeX-default-paper-type)
-       (if (member "landscape" opts) "r" "")))))
+       (if (YaTeX-member "landscape" opts) (if YaTeX-dos "L" "r") "")))))
 
 (defvar YaTeX-preview-command-history nil
   "Holds minibuffer history of preview command.")
@@ -410,23 +428,27 @@ PROC should be process identifier."
 (defun YaTeX-preview (preview-command preview-file)
   "Execute xdvi (or other) to tex-preview."
   (interactive
-   (list
-    (read-string-with-history
-     "Preview command: "
-     (YaTeX-replace-format
-      (or (YaTeX-get-builtin "PREVIEW") dvi2-command)
-      "p" (concat "-paper " (YaTeX-get-paper-type)))
-     'YaTeX-preview-command-history)
-    (read-string-with-history
-     "Preview file[.dvi]: "
-     (if (get 'dvi2-command 'region)
-	 (substring YaTeX-texput-file
-		    0 (rindex YaTeX-texput-file ?.))
-       (YaTeX-get-preview-file-name))
-     'YaTeX-preview-file-history)))
+   (let* ((command (read-string-with-history
+		    "Preview command: "
+		    (YaTeX-replace-format
+		     (or (YaTeX-get-builtin "PREVIEW") dvi2-command)
+		     "p" (format (cond
+				  (YaTeX-dos "-y:%s")
+				  (t "-paper %s"))
+				 (YaTeX-get-paper-type)))
+		    'YaTeX-preview-command-history))
+	  (file (read-string-with-history
+		 "Preview file: "
+		 (if (get 'dvi2-command 'region)
+		     (substring YaTeX-texput-file
+				0 (rindex YaTeX-texput-file ?.))
+		   (YaTeX-get-preview-file-name command))
+		 'YaTeX-preview-file-history)))
+     (list command file)))
   (setq dvi2-command preview-command)	;`dvi2command' is buffer local
   (save-excursion
     (YaTeX-visit-main t)
+    (if YaTeX-dos (setq preview-file (expand-file-name preview-file)))
     (let ((pbuffer "*dvi-preview*") (dir default-directory))
       (YaTeX-showup-buffer
        pbuffer (function (lambda (x) (nth 3 (window-edges x)))))
@@ -450,9 +472,10 @@ PROC should be process identifier."
 		      (concat preview-command " " preview-file)))
        (t				;if UNIX
 	(set-process-buffer
-	 (start-process "preview" "*dvi-preview*" shell-file-name
-			YaTeX-shell-command-option
-			(concat preview-command " " preview-file))
+	 (let ((process-connection-type nil))
+	   (start-process "preview" "*dvi-preview*" shell-file-name
+			  YaTeX-shell-command-option
+			  (concat preview-command " " preview-file)))
 	 (get-buffer pbuffer))
 	(message
 	 (concat "Starting " preview-command
@@ -500,7 +523,7 @@ by region."
 error or warning lines in reverse order."
   (interactive)
   (let ((cur-buf (buffer-name)) (cur-win (selected-window))
-	error-line typeset-win error-buffer error-win)
+	b0 errorp error-line typeset-win error-buffer error-win)
     (if (null (get-buffer YaTeX-typeset-buffer))
 	(error "There is no typesetting buffer."))
     (YaTeX-showup-buffer YaTeX-typeset-buffer nil t)
@@ -509,10 +532,10 @@ error or warning lines in reverse order."
 	 (concat "\\(" latex-error-regexp "\\)\\|\\("
 		 latex-warning-regexp "\\)")
 	 nil t)
-	nil
+	(setq errorp (match-beginning 1))
       (select-window cur-win)
       (error "No more errors on %s" cur-buf))
-    (goto-char (match-beginning 0))
+    (goto-char (setq b0 (match-beginning 0)))
     (skip-chars-forward "^0-9" (match-end 0))
     (setq error-line
 	  (string-to-int
@@ -534,13 +557,13 @@ error or warning lines in reverse order."
     (setq error-win (selected-window))
     (goto-line error-line)
     (message "LaTeX %s in `%s' on line: %d."
-	     (if (match-beginning 1) "error" "warning")
+	     (if errorp "error" "warning")
 	     error-buffer error-line)
     (select-window typeset-win)
     (skip-chars-backward "0-9")
     (recenter (/ (window-height) 2))
     (sit-for 1)
-    (goto-char (match-beginning 0))
+    (goto-char b0)
     (select-window error-win)))
 
 (defun YaTeX-jump-error-line ()
@@ -642,18 +665,28 @@ error or warning lines in reverse order."
 	(delete-matching-lines "^\\\\nonstopmode\\{\\}%_YaTeX_%$")
 	(widen))))
 
-(defun YaTeX-get-preview-file-name ()
+(defvar YaTeX-dvi2-command-ext-alist
+ '(("[agx]dvi\\|dviout" . ".dvi")
+   ("ghostview\\|gv" . ".ps")
+   ("acroread\\|pdf\\|Preview\\|TeXShop\\|Skim" . ".pdf")))
+
+(defun YaTeX-get-preview-file-name (&optional preview-command)
   "Get file name to preview by inquiring YaTeX-get-latex-command"
+  (if (null preview-command) (setq preview-command dvi2-command))
   (let* ((latex-cmd (YaTeX-get-latex-command t))
 	 (rin (rindex latex-cmd ? ))
 	 (fname (if (> rin -1) (substring latex-cmd (1+ rin)) ""))
+	 (r (YaTeX-assoc-regexp preview-command YaTeX-dvi2-command-ext-alist))
+	 (ext (if r (cdr r) ""))
 	 (period))
-    (if (string= fname "")
+    (concat
+     (if (string= fname "")
 	(setq fname (substring (file-name-nondirectory
 				(buffer-file-name))
 			       0 -4))
       (setq period (rindex fname ?.))
-      (setq fname (substring fname 0 (if (eq -1 period) nil period))))))
+      (setq fname (substring fname 0 (if (eq -1 period) nil period))))
+     ext)))
 
 (defun YaTeX-get-latex-command (&optional switch)
   "Specify the latex-command name and its argument.
@@ -669,29 +702,38 @@ is non-nil, then
 	\"jlatex main.tex\"
 
 will be given to the shell."
-  (let (magic command target)
+  (let (parent tparent magic)
     (setq parent
 	  (cond
-	   (YaTeX-parent-file YaTeX-parent-file)
+	   (YaTeX-parent-file
+	    (if YaTeX-dos (expand-file-name YaTeX-parent-file)
+	      YaTeX-parent-file))
 	   (t (save-excursion
 		(YaTeX-visit-main t)
 		(file-name-nondirectory (buffer-file-name)))))
-	  magic (YaTeX-get-builtin "!"))
-    (cond
-     (magic
-      (cond
-       (switch (if (string-match "\\s " magic) magic
-		 (concat magic " " parent)))
-       (t (concat (substring magic 0 (string-match "\\s " magic)) " "))))
-     (t (concat tex-command " " (if switch parent))))))
+	  magic (YaTeX-get-builtin "!")
+	  tparent (file-name-nondirectory parent))
+    (YaTeX-replace-formats
+     (cond
+      (magic
+       (cond
+	(switch (if (string-match "\\s " magic) magic
+		  (concat magic " " parent)))
+	(t (concat (substring magic 0 (string-match "\\s " magic)) " "))))
+      (t (concat tex-command " " (if switch parent))))
+     (list (cons "f" tparent)
+	   (cons "r" (substring tparent 0 (rindex tparent ?.)))))))
 
 (defvar YaTeX-lpr-command-history nil
   "Holds command line history of YaTeX-lpr.")
 (put 'YaTeX-lpr-command-history 'no-default t)
+(defvar YaTeX-lpr-ask-page-range-default t)
 (defun YaTeX-lpr (arg)
-  "Print out.  If prefix arg ARG is non nil, call print driver without
+  "Print out.
+If prefix arg ARG is non nil, call print driver without
 page range description."
   (interactive "P")
+  (or YaTeX-lpr-ask-page-range-default (setq arg (not arg)))
   (let*((cmd (or (YaTeX-get-builtin "LPR") dviprint-command-format))
 	from to (lbuffer "*dvi-printing*") dir)
     (setq
@@ -745,8 +787,9 @@ page range description."
 		      YaTeX-shell-command-option cmd))
        (t
 	(set-process-buffer
-	 (start-process "print" "*dvi-printing*" shell-file-name
-			YaTeX-shell-command-option cmd)
+	 (let ((process-connection-type nil))
+	   (start-process "print" "*dvi-printing*" shell-file-name
+			  YaTeX-shell-command-option cmd))
 	 (get-buffer lbuffer))
 	(message "Starting printing command: %s..." cmd))))))
 
@@ -799,8 +842,9 @@ SETBUF is t(Use it only from Emacs-Lisp program)."
        ;;((and main-file (YaTeX-switch-to-buffer main-file setbuf)))
        ((and main-file
 	     (file-exists-p (setq main-file (concat "../" main-file)))
-	     (y-or-n-p (concat (expand-file-name main-file)
-			       " is main file?:")))
+	     (or b-in
+		 (y-or-n-p (concat (expand-file-name main-file)
+				   " is main file?:"))))
 	(setq YaTeX-parent-file main-file)
 	;(YaTeX-switch-to-buffer main-file setbuf)
 	(funcall ff main-file)
@@ -837,7 +881,7 @@ SETBUF is t(Use it only from Emacs-Lisp program)."
     (if (and (re-search-forward
 	      (concat "^" (regexp-quote (concat "%#" key))) nil t)
 	     (not (eolp)))
-	(buffer-substring
+	(YaTeX-buffer-substring
 	 (progn (skip-chars-forward " 	" (point-end-of-line))(point))
 	 (point-end-of-line))
       nil)))
